(** ast.mli - This file defines the types for an Abstract Syntax Tree that is produced by
   the parser.
   Tabari Alexander tha2105
   COMS W4115
*)

(** The set of operators for an arithmetic op *)
type operator = Add | Sub | Mul | Div | Cons

(** The set of operators for a relation op *) 
type relop = Eq | Ne | Lt | Gt | LtEq | GtEq

(** The set of operators for a logic op *)
type logicop = And | Or

(** Expressions that mostly map to the expressions in the language's grammar. *)
type expr = 
    IntLit of int
  | AtomLit of string
  | EmptyList
  | StrLit of string
  | Call of string * expr list
  | BinOp of expr * operator * expr
  | RelOp of expr * relop * expr
  | LogOp of expr * logicop * expr
  | Var of string
  | RetVar of string
  | Asn of string * expr
  | IfExpr of expr * expr * expr
  | Not of expr
  | Neg of expr

(** Patterns are used in predicate definition arguments. *)
type ptrn = 
    IntPtrn of int
  | AtomPtrn of string
  | VarPtrn of string
  | ConsPtrn of ptrn * ptrn
  | EmptyPtrn

type pred = Pred of ptrn list * expr option

type stmt = 
    ExprStmt of expr
  | IfStmt of expr * stmt * stmt
  | Block of stmt list
  | While of expr * stmt
  | Until of expr * stmt

type decl = {name : string; arity : int; decls : pred list}

type prog =
    Prog of decl list * stmt

