(** clogic.ml - The compiler front-end. 
   Tabari Alexander tha2105
   COMS W4115
*)
open Ast

let _ =
try
  (* The following are references used when parsing the options in the compiler*)
  let print_ir = ref false in
  let compile_only = ref true in
  let infile = ref None in
  let outfile = ref None in 
  (* Parse the options.  The return value is a Unit, it can be ignored. *)
  let _ =
    let usage = "Usage: cl [options] file\nOptions:" in
    let options = 
      [("-c", Arg.Unit(fun () -> compile_only := true), "\tCompile and assemble the code to be interpreted by clvm.");
       ("-S", Arg.Unit(fun () -> print_ir := true), "\tPrint the IR to a file.");
       ("-o", Arg.String(fun o -> outfile := Some o), "<filename>\tPlace the output into <filename>.")]
    in
    Arg.parse options (fun fname -> infile := Some(fname)) usage in
  (* Determine the input source.  If no file is specified, then stdin is used. *)
  let instream = match !infile with Some(f) -> open_in f | _ -> stdin in
  let prefix = match !infile with 
      Some(f) -> String.sub f 0 (String.rindex f '.')
    | None -> "a" in 
  let outfile = match !outfile with 
      Some(f) -> f
    | None -> prefix ^ ".clo" in
  let irfile = prefix ^ ".ir" in
  let lexbuf = Lexing.from_channel instream in
  (* Finally, perform front-end operations.  This will tokenize, parse, and compile 
     the input *)
  let ir = Translator.compile(Sast.translate(Parser.program Scanner.token lexbuf))
  in
  (* Perform back-end processing.  If it was specified to print the IR to a file,
     then it will be done before compiling the IR into bytecode. *)
  (if !print_ir then Opcodes.print_ir irfile ir);
  if !compile_only then Opcodes.assemble outfile ir
with
  Scanner.ScannerError x -> prerr_endline x; exit 2
| Sast.SyntaxError x -> prerr_endline x; exit 2
| Failure x -> prerr_endline("Unexpected error: " ^ x); exit 2
