type opcode = 
    AddOp
  | SubOp
  | MulOp
  | DivOp
  | ConsOp
  | NotOp
  | PushInt of int
  | PushAtom of string
  | PushEmpty
  | PushBool of int
  | PushString of string
  | PushRef of int
  | Head
  | Tail
  | Pop
  | PushFrame of int
  | Store of int
  | StoreVal of int
  | ClearVal of int
  | CallOp of string
  | BuiltIn of int
  | Ret
  | Load of int
  | EqOp
  | AsnEqOp
  | NeOp
  | LtOp
  | LtEqOp
  | GtOp
  | GtEqOp
  | TestInt
  | TestAtom
  | TestBool
  | TestList
  | TestString
  | Jnz of string
  | Jz of string
  | Goto of string
  | PopFrame
  | TodoOp
  | Halt
  | Label of string
  | StartBT of int
  | StopBT of int

val int_of_op : opcode -> int

val op_length : opcode -> int

val print_ir : string -> opcode list -> unit

val assemble : string -> opcode list -> unit
(*
The main execution block should be like so:
    PUSHFRAME
    PUSHINT 0
    STORE 0
    CALL 1 /* 1 is the symbol for test/1 and R0 is placed on the stack */
R0: POP
    HALT

Stack before call:
[ 0 ]

A truth, if executed, pretty much amounts to:
    PUSHBOOL 1

Finding the truth in this scenario:
    LOAD 0
    PUSHINT 0
    CMP
    JNZ L0
    PUSHBOOL 1
    POPFRAME /* pops and goes to the return address on the stack */
L0: PUSHBOOL 0
    POPFRAME
*)
