type token =
  | EOF
  | LPAREN
  | RPAREN
  | LBRACE
  | RBRACE
  | SEMICOLON
  | COMMA
  | IMPLY
  | IN
  | LBRACK
  | RBRACK
  | CONS
  | ASSIGN
  | PLUS
  | MINUS
  | TIMES
  | DIVIDE
  | TYPE
  | THEN
  | ELSE
  | NOT
  | AND
  | OR
  | LT
  | GT
  | LTEQ
  | GTEQ
  | EQ
  | NE
  | EMPTYLIST
  | UNTIL
  | WHILE
  | VARIABLE of (string)
  | ID of (string)
  | STRINGLIT of (string)
  | INTLIT of (int)

open Parsing;;
# 6 "parser.mly"
  open Ast

  module StringMap = Map.Make(String)

  type decl = {nargs:int; truths:pred list; preds:pred list}

  let rec ptrn_of_lit = function
      IntLit(x) -> IntPtrn(x)
    | AtomLit(x) -> AtomPtrn(x)
    | Var(x) -> VarPtrn(x)
    | EmptyList -> EmptyPtrn
    | _ -> raise(Failure "Unexpected literal in pattern")

  let predefines = 
    [("true", 0, Pred([],
        Some(Call("__true", []))));
     ("false", 0, Pred([],
        Some(Call("__false", []))));
     ("neg", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(Call("__integer", [Var "X"]), And, Call("__neg",[Var "X"; Var "Y"])))));
     ("tointeger", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(Call("__string", [Var "X"]), And, Call("__tointeger",[Var "X"; Var "Y"])))));
     ("tostring", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(Call("__neg",[Var "X"; Var "Y"]))));
     ("exit", 1, Pred([VarPtrn "X"], 
        Some(Call("__exit",[Var "X"]))));
     ("exit", 0, Pred([], 
        Some(Call("__exit",[IntLit 0]))));
     ("true", 1, Pred([VarPtrn "X"], 
        Some(Call("__true",[Var "X"]))));
     ("integer", 1, Pred([VarPtrn "X"], 
        Some(Call("__integer",[Var "X"]))));
     ("string", 1, Pred([VarPtrn "X"], 
        Some(Call("__string",[Var "X"]))));
     ("boolean", 1, Pred([VarPtrn "X"], 
        Some(Call("__boolean",[Var "X"]))));
     ("atom", 1, Pred([VarPtrn "X"], 
        Some(Call("__atom",[Var "X"]))));     
     ("list", 1, Pred([VarPtrn "X"], 
        Some(Call("__list",[Var "X"]))));
     ("list", 1, Pred([EmptyPtrn], None));
     ("not", 1, Pred([VarPtrn "X"], 
        Some(Call("__not",[Call("__true",[Var "X"])]))));
     ("gt", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__gt",[Var "X"; Var "Y"])))));
     ("lt", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__lt",[Var "X"; Var "Y"])))));
     ("ge", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__ge",[Var "X"; Var "Y"])))));
     ("le", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__le",[Var "X"; Var "Y"])))));
     ("eq", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(Call("__eq",[Var "X"; Var "Y"]))));
     ("ne", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(Call("__ne",[Var "X"; Var "Y"]))));
     ("cons", 3, Pred([VarPtrn "X"; VarPtrn "Y"; VarPtrn "Z"], 
        Some(LogOp(LogOp(RelOp(Var "Y",Eq, EmptyList),Or,Call("__list", [Var "Y"])), And, Call("__cons",[Var "X"; Var "Y"; Var "Z"])))));
     ("mult", 3, Pred([VarPtrn "X"; VarPtrn "Y"; VarPtrn "Z"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__mult",[Var "X"; Var "Y"; Var "Z"])))));
     ("div", 3, Pred([VarPtrn "X"; VarPtrn "Y"; VarPtrn "Z"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__div",[Var "X"; Var "Y"; Var "Z"])))));
     ("div", 3, Pred([VarPtrn "X"; IntPtrn 0; VarPtrn "Z"],
        Some(LogOp(Call("__print", [StrLit "Divide by zero error"]), And, Call("__exit", [IntLit 1])))));
     ("add", 3, Pred([VarPtrn "X"; VarPtrn "Y"; VarPtrn "Z"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__add",[Var "X"; Var "Y"; Var "Z"])))));
     ("sub", 3, Pred([VarPtrn "X"; VarPtrn "Y"; VarPtrn "Z"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__sub",[Var "X"; Var "Y"; Var "Z"])))));
     ("print", 1, Pred([VarPtrn "X"], 
        Some(Call("__print",[Var "X"]))));
     ("write", 1, Pred([VarPtrn "X"], 
        Some(Call("__write", [Var "X"]))));
     ("write", 1, Pred([ConsPtrn(VarPtrn "H", VarPtrn "T")],
        Some(LogOp(Call("write", [Var "H"]), And, Call("write", [Var "T"])))));
     ("write", 1, Pred([VarPtrn "X"],
        Some(LogOp(Call("__string", [Var "X"]), And, Call("__write", [Var "X"])))))] 

  let add_decl sm = function
    (n, a, p) -> 
      let name = n ^ "/" ^ (string_of_int a) in
      let r = 
        try
          StringMap.find name sm
        with
          Not_found -> {nargs = a; truths=[]; preds=[]} in
      let r = 
        (match p with Pred(_, Some e) -> { r with preds = p::r.preds }
          | Pred(_, None) -> { r with truths = p::r.truths })
      in
      StringMap.add name r sm
       
# 132 "parser.ml"
let yytransl_const = [|
    0 (* EOF *);
  257 (* LPAREN *);
  258 (* RPAREN *);
  259 (* LBRACE *);
  260 (* RBRACE *);
  261 (* SEMICOLON *);
  262 (* COMMA *);
  263 (* IMPLY *);
  264 (* IN *);
  265 (* LBRACK *);
  266 (* RBRACK *);
  267 (* CONS *);
  268 (* ASSIGN *);
  269 (* PLUS *);
  270 (* MINUS *);
  271 (* TIMES *);
  272 (* DIVIDE *);
  273 (* TYPE *);
  274 (* THEN *);
  275 (* ELSE *);
  276 (* NOT *);
  277 (* AND *);
  278 (* OR *);
  279 (* LT *);
  280 (* GT *);
  281 (* LTEQ *);
  282 (* GTEQ *);
  283 (* EQ *);
  284 (* NE *);
  285 (* EMPTYLIST *);
  286 (* UNTIL *);
  287 (* WHILE *);
    0|]

let yytransl_block = [|
  288 (* VARIABLE *);
  289 (* ID *);
  290 (* STRINGLIT *);
  291 (* INTLIT *);
    0|]

let yylhs = "\255\255\
\001\000\004\000\004\000\004\000\004\000\005\000\005\000\005\000\
\005\000\005\000\005\000\008\000\011\000\011\000\011\000\012\000\
\012\000\012\000\013\000\013\000\013\000\014\000\014\000\015\000\
\015\000\015\000\015\000\015\000\015\000\015\000\016\000\016\000\
\017\000\017\000\018\000\018\000\006\000\010\000\010\000\019\000\
\019\000\020\000\020\000\007\000\007\000\009\000\009\000\002\000\
\002\000\021\000\021\000\022\000\022\000\024\000\025\000\025\000\
\023\000\023\000\026\000\026\000\027\000\027\000\027\000\027\000\
\028\000\028\000\028\000\028\000\028\000\003\000\003\000\029\000\
\029\000\030\000\030\000\030\000\030\000\030\000\000\000"

let yylen = "\002\000\
\002\000\001\000\001\000\001\000\001\000\001\000\001\000\003\000\
\003\000\001\000\007\000\004\000\001\000\002\000\002\000\001\000\
\003\000\003\000\001\000\003\000\003\000\001\000\003\000\001\000\
\003\000\003\000\003\000\003\000\003\000\003\000\001\000\003\000\
\001\000\003\000\001\000\003\000\001\000\000\000\001\000\001\000\
\003\000\001\000\002\000\001\000\003\000\000\000\001\000\000\000\
\001\000\001\000\002\000\002\000\002\000\004\000\001\000\003\000\
\006\000\006\000\000\000\001\000\001\000\003\000\003\000\003\000\
\001\000\003\000\003\000\003\000\003\000\003\000\002\000\001\000\
\002\000\002\000\005\000\001\000\004\000\004\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\000\000\079\000\000\000\000\000\050\000\000\000\
\000\000\000\000\000\000\001\000\051\000\052\000\053\000\005\000\
\000\000\003\000\004\000\002\000\000\000\000\000\000\000\000\000\
\061\000\000\000\071\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\076\000\006\000\013\000\000\000\010\000\016\000\
\000\000\000\000\024\000\000\000\033\000\000\000\000\000\000\000\
\072\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\044\000\000\000\007\000\014\000\015\000\000\000\000\000\000\000\
\000\000\074\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\070\000\073\000\000\000\000\000\066\000\000\000\068\000\000\000\
\000\000\062\000\000\000\000\000\063\000\008\000\000\000\000\000\
\009\000\000\000\000\000\000\000\000\000\042\000\000\000\000\000\
\040\000\000\000\017\000\018\000\023\000\000\000\000\000\025\000\
\026\000\027\000\028\000\029\000\030\000\034\000\000\000\057\000\
\058\000\000\000\045\000\078\000\077\000\043\000\012\000\000\000\
\000\000\000\000\041\000\075\000\047\000\000\000\011\000"

let yydgoto = "\002\000\
\004\000\005\000\035\000\036\000\037\000\038\000\058\000\039\000\
\134\000\103\000\040\000\041\000\042\000\043\000\044\000\045\000\
\046\000\047\000\104\000\105\000\006\000\007\000\008\000\009\000\
\022\000\023\000\024\000\025\000\048\000\049\000"

let yysindex = "\006\000\
\239\254\000\000\033\255\000\000\018\255\239\254\000\000\035\255\
\042\255\046\255\054\255\000\000\000\000\000\000\000\000\000\000\
\045\255\000\000\000\000\000\000\048\255\013\255\059\255\060\255\
\000\000\131\255\000\000\131\255\138\255\138\255\131\255\131\255\
\079\255\093\255\000\000\000\000\000\000\004\255\000\000\000\000\
\254\254\100\255\000\000\106\000\000\000\051\255\073\255\089\255\
\000\000\081\000\093\000\099\255\046\255\108\255\046\255\006\255\
\000\000\020\255\000\000\000\000\000\000\102\255\128\255\138\255\
\003\255\000\000\096\255\138\255\138\255\138\255\138\255\138\255\
\138\255\138\255\138\255\138\255\138\255\138\255\131\255\131\255\
\000\000\000\000\000\000\048\255\000\000\000\000\000\000\131\255\
\048\255\000\000\131\255\048\255\000\000\000\000\131\255\131\255\
\000\000\096\255\096\255\106\000\069\255\000\000\132\255\129\255\
\000\000\117\255\000\000\000\000\000\000\254\254\254\254\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\051\255\000\000\
\000\000\118\255\000\000\000\000\000\000\000\000\000\000\003\255\
\096\255\131\255\000\000\000\000\000\000\141\255\000\000"

let yyrindex = "\000\000\
\145\255\000\000\000\000\000\000\000\000\146\255\000\000\000\000\
\000\000\142\255\000\000\000\000\000\000\000\000\000\000\000\000\
\025\255\000\000\000\000\000\000\058\255\000\000\000\000\148\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\172\255\199\255\000\000\000\000\000\000\000\000\000\000\000\000\
\226\255\051\000\000\000\078\000\000\000\023\255\136\255\000\000\
\000\000\000\000\000\000\151\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\155\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\065\255\000\000\000\000\106\255\000\000\000\000\
\071\255\000\000\000\000\094\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\084\000\000\000\000\000\000\000\157\255\
\000\000\000\000\000\000\000\000\000\000\253\255\024\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\102\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\159\255\000\000\000\000\000\000\000\000\000\000"

let yygindex = "\000\000\
\000\000\000\000\148\000\249\255\000\000\230\255\000\000\000\000\
\000\000\000\000\237\255\238\255\000\000\245\000\098\000\089\000\
\095\000\000\000\000\000\041\000\000\000\170\000\000\000\000\000\
\000\000\000\000\000\000\085\001\000\000\209\255"

let yytablesize = 396
let yytable = "\056\000\
\082\000\057\000\021\000\026\000\062\000\063\000\001\000\094\000\
\066\000\060\000\061\000\028\000\068\000\069\000\052\000\003\000\
\029\000\101\000\053\000\106\000\011\000\067\000\030\000\095\000\
\035\000\096\000\065\000\035\000\035\000\097\000\065\000\016\000\
\035\000\010\000\033\000\034\000\019\000\020\000\102\000\014\000\
\035\000\035\000\084\000\084\000\035\000\089\000\015\000\092\000\
\107\000\108\000\124\000\125\000\110\000\111\000\026\000\050\000\
\011\000\027\000\051\000\055\000\054\000\120\000\028\000\055\000\
\121\000\055\000\067\000\029\000\122\000\123\000\067\000\079\000\
\056\000\030\000\016\000\005\000\056\000\017\000\018\000\019\000\
\020\000\132\000\016\000\031\000\032\000\033\000\034\000\019\000\
\020\000\026\000\064\000\011\000\081\000\065\000\080\000\064\000\
\026\000\028\000\011\000\064\000\126\000\102\000\029\000\133\000\
\028\000\088\000\098\000\069\000\030\000\029\000\070\000\069\000\
\071\000\072\000\091\000\030\000\005\000\016\000\031\000\032\000\
\033\000\034\000\019\000\020\000\016\000\031\000\032\000\033\000\
\034\000\019\000\020\000\026\000\099\000\127\000\128\000\129\000\
\130\000\037\000\026\000\028\000\037\000\037\000\135\000\059\000\
\029\000\037\000\028\000\048\000\049\000\060\000\030\000\029\000\
\012\000\037\000\037\000\054\000\038\000\030\000\039\000\016\000\
\046\000\100\000\033\000\034\000\019\000\020\000\016\000\118\000\
\131\000\059\000\034\000\019\000\020\000\007\000\119\000\013\000\
\007\000\007\000\000\000\000\000\000\000\007\000\007\000\000\000\
\007\000\007\000\007\000\007\000\000\000\007\000\007\000\000\000\
\007\000\007\000\007\000\007\000\007\000\007\000\007\000\007\000\
\003\000\000\000\000\000\003\000\003\000\000\000\000\000\000\000\
\003\000\003\000\000\000\003\000\003\000\003\000\003\000\000\000\
\003\000\003\000\000\000\003\000\003\000\003\000\003\000\003\000\
\003\000\003\000\003\000\019\000\000\000\000\000\019\000\019\000\
\000\000\000\000\000\000\019\000\019\000\000\000\019\000\019\000\
\000\000\000\000\000\000\019\000\019\000\000\000\019\000\019\000\
\019\000\019\000\019\000\019\000\019\000\019\000\020\000\000\000\
\000\000\020\000\020\000\000\000\000\000\000\000\020\000\020\000\
\000\000\020\000\020\000\000\000\000\000\000\000\020\000\020\000\
\000\000\020\000\020\000\020\000\020\000\020\000\020\000\020\000\
\020\000\021\000\000\000\000\000\021\000\021\000\000\000\000\000\
\000\000\021\000\021\000\000\000\021\000\021\000\000\000\000\000\
\000\000\021\000\021\000\000\000\021\000\021\000\021\000\021\000\
\021\000\021\000\021\000\021\000\022\000\000\000\000\000\022\000\
\022\000\000\000\109\000\000\000\022\000\112\000\113\000\114\000\
\115\000\116\000\117\000\000\000\022\000\022\000\000\000\022\000\
\022\000\022\000\022\000\022\000\022\000\022\000\022\000\031\000\
\000\000\000\000\031\000\031\000\000\000\032\000\000\000\031\000\
\032\000\032\000\000\000\000\000\000\000\032\000\000\000\031\000\
\031\000\000\000\031\000\031\000\000\000\032\000\032\000\036\000\
\032\000\032\000\036\000\036\000\000\000\083\000\000\000\036\000\
\017\000\018\000\019\000\020\000\000\000\000\000\000\000\036\000\
\036\000\086\000\000\000\036\000\017\000\018\000\019\000\020\000\
\073\000\074\000\075\000\076\000\077\000\078\000\085\000\087\000\
\000\000\090\000\000\000\093\000"

let yycheck = "\026\000\
\048\000\028\000\010\000\001\001\031\000\032\000\001\000\002\001\
\005\001\029\000\030\000\009\001\015\001\016\001\002\001\033\001\
\014\001\015\001\006\001\067\000\003\001\018\001\020\001\018\001\
\002\001\006\001\002\001\005\001\006\001\010\001\006\001\029\001\
\010\001\001\001\032\001\033\001\034\001\035\001\065\000\005\001\
\018\001\019\001\050\000\051\000\022\001\053\000\005\001\055\000\
\068\000\069\000\098\000\099\000\071\000\072\000\001\001\011\001\
\003\001\004\001\011\001\002\001\002\001\088\000\009\001\006\001\
\091\000\006\001\002\001\014\001\095\000\096\000\006\001\021\001\
\002\001\020\001\029\001\011\001\006\001\032\001\033\001\034\001\
\035\001\129\000\029\001\030\001\031\001\032\001\033\001\034\001\
\035\001\001\001\012\001\003\001\004\001\001\001\022\001\002\001\
\001\001\009\001\003\001\006\001\032\001\128\000\014\001\130\000\
\009\001\007\001\005\001\002\001\020\001\014\001\011\001\006\001\
\013\001\014\001\007\001\020\001\011\001\029\001\030\001\031\001\
\032\001\033\001\034\001\035\001\029\001\030\001\031\001\032\001\
\033\001\034\001\035\001\001\001\005\001\002\001\006\001\019\001\
\019\001\002\001\001\001\009\001\005\001\006\001\002\001\002\001\
\014\001\010\001\009\001\003\001\003\001\002\001\020\001\014\001\
\005\000\018\001\019\001\005\001\002\001\020\001\002\001\029\001\
\002\001\064\000\032\001\033\001\034\001\035\001\029\001\079\000\
\128\000\032\001\033\001\034\001\035\001\002\001\080\000\006\000\
\005\001\006\001\255\255\255\255\255\255\010\001\011\001\255\255\
\013\001\014\001\015\001\016\001\255\255\018\001\019\001\255\255\
\021\001\022\001\023\001\024\001\025\001\026\001\027\001\028\001\
\002\001\255\255\255\255\005\001\006\001\255\255\255\255\255\255\
\010\001\011\001\255\255\013\001\014\001\015\001\016\001\255\255\
\018\001\019\001\255\255\021\001\022\001\023\001\024\001\025\001\
\026\001\027\001\028\001\002\001\255\255\255\255\005\001\006\001\
\255\255\255\255\255\255\010\001\011\001\255\255\013\001\014\001\
\255\255\255\255\255\255\018\001\019\001\255\255\021\001\022\001\
\023\001\024\001\025\001\026\001\027\001\028\001\002\001\255\255\
\255\255\005\001\006\001\255\255\255\255\255\255\010\001\011\001\
\255\255\013\001\014\001\255\255\255\255\255\255\018\001\019\001\
\255\255\021\001\022\001\023\001\024\001\025\001\026\001\027\001\
\028\001\002\001\255\255\255\255\005\001\006\001\255\255\255\255\
\255\255\010\001\011\001\255\255\013\001\014\001\255\255\255\255\
\255\255\018\001\019\001\255\255\021\001\022\001\023\001\024\001\
\025\001\026\001\027\001\028\001\002\001\255\255\255\255\005\001\
\006\001\255\255\070\000\255\255\010\001\073\000\074\000\075\000\
\076\000\077\000\078\000\255\255\018\001\019\001\255\255\021\001\
\022\001\023\001\024\001\025\001\026\001\027\001\028\001\002\001\
\255\255\255\255\005\001\006\001\255\255\002\001\255\255\010\001\
\005\001\006\001\255\255\255\255\255\255\010\001\255\255\018\001\
\019\001\255\255\021\001\022\001\255\255\018\001\019\001\002\001\
\021\001\022\001\005\001\006\001\255\255\029\001\255\255\010\001\
\032\001\033\001\034\001\035\001\255\255\255\255\255\255\018\001\
\019\001\029\001\255\255\022\001\032\001\033\001\034\001\035\001\
\023\001\024\001\025\001\026\001\027\001\028\001\050\000\051\000\
\255\255\053\000\255\255\055\000"

let yynames_const = "\
  EOF\000\
  LPAREN\000\
  RPAREN\000\
  LBRACE\000\
  RBRACE\000\
  SEMICOLON\000\
  COMMA\000\
  IMPLY\000\
  IN\000\
  LBRACK\000\
  RBRACK\000\
  CONS\000\
  ASSIGN\000\
  PLUS\000\
  MINUS\000\
  TIMES\000\
  DIVIDE\000\
  TYPE\000\
  THEN\000\
  ELSE\000\
  NOT\000\
  AND\000\
  OR\000\
  LT\000\
  GT\000\
  LTEQ\000\
  GTEQ\000\
  EQ\000\
  NE\000\
  EMPTYLIST\000\
  UNTIL\000\
  WHILE\000\
  "

let yynames_block = "\
  VARIABLE\000\
  ID\000\
  STRINGLIT\000\
  INTLIT\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'decls_opt) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'block) in
    Obj.repr(
# 110 "parser.mly"
      ( Prog(StringMap.fold (fun n r l -> {name=n; decls = List.rev_append r.truths r.preds; arity = r.nargs}::l) _1 [],_2) )
# 423 "parser.ml"
               : Ast.prog))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 113 "parser.mly"
           ( IntLit _1 )
# 430 "parser.ml"
               : 'literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 114 "parser.mly"
       ( AtomLit _1 )
# 437 "parser.ml"
               : 'literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 115 "parser.mly"
              ( StrLit _1 )
# 444 "parser.ml"
               : 'literal))
; (fun __caml_parser_env ->
    Obj.repr(
# 116 "parser.mly"
              ( EmptyList )
# 450 "parser.ml"
               : 'literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'literal) in
    Obj.repr(
# 119 "parser.mly"
            ( _1 )
# 457 "parser.ml"
               : 'primary_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 120 "parser.mly"
             ( Var _1  )
# 464 "parser.ml"
               : 'primary_expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 121 "parser.mly"
                       ( _2 )
# 471 "parser.ml"
               : 'primary_expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'expr_list) in
    Obj.repr(
# 122 "parser.mly"
                            ( List.fold_left (fun e li -> BinOp(li, Cons, e)) EmptyList _2 )
# 478 "parser.ml"
               : 'primary_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'query) in
    Obj.repr(
# 123 "parser.mly"
          ( _1 )
# 485 "parser.ml"
               : 'primary_expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : 'expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'expr) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : 'expr_opt) in
    Obj.repr(
# 124 "parser.mly"
                                               ( IfExpr(_2, _4, _6) )
# 494 "parser.ml"
               : 'primary_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'query_args_opt) in
    Obj.repr(
# 127 "parser.mly"
                                    ( Call(_1, List.rev _3) )
# 502 "parser.ml"
               : 'query))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'primary_expr) in
    Obj.repr(
# 130 "parser.mly"
                 ( _1 )
# 509 "parser.ml"
               : 'unary_expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'unary_expr) in
    Obj.repr(
# 131 "parser.mly"
                     ( Neg _2 )
# 516 "parser.ml"
               : 'unary_expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'unary_expr) in
    Obj.repr(
# 132 "parser.mly"
                   ( Not _2 )
# 523 "parser.ml"
               : 'unary_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unary_expr) in
    Obj.repr(
# 135 "parser.mly"
               ( _1 )
# 530 "parser.ml"
               : 'mult_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'mult_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'unary_expr) in
    Obj.repr(
# 136 "parser.mly"
                               ( BinOp(_1, Mul, _3) )
# 538 "parser.ml"
               : 'mult_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'mult_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'unary_expr) in
    Obj.repr(
# 137 "parser.mly"
                                ( BinOp(_1, Div, _3) )
# 546 "parser.ml"
               : 'mult_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'mult_expr) in
    Obj.repr(
# 140 "parser.mly"
              ( _1 )
# 553 "parser.ml"
               : 'add_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'add_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'mult_expr) in
    Obj.repr(
# 141 "parser.mly"
                            ( BinOp(_1, Add, _3) )
# 561 "parser.ml"
               : 'add_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'add_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'mult_expr) in
    Obj.repr(
# 142 "parser.mly"
                             ( BinOp(_1, Sub, _3) )
# 569 "parser.ml"
               : 'add_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'add_expr) in
    Obj.repr(
# 145 "parser.mly"
             ( _1 )
# 576 "parser.ml"
               : 'cons_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'add_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cons_expr) in
    Obj.repr(
# 146 "parser.mly"
                            ( BinOp(_1, Cons, _3) )
# 584 "parser.ml"
               : 'cons_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'cons_expr) in
    Obj.repr(
# 149 "parser.mly"
              ( _1 )
# 591 "parser.ml"
               : 'rel_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'rel_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cons_expr) in
    Obj.repr(
# 150 "parser.mly"
                          ( RelOp(_1, Lt, _3) )
# 599 "parser.ml"
               : 'rel_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'rel_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cons_expr) in
    Obj.repr(
# 151 "parser.mly"
                          ( RelOp(_1, Gt, _3) )
# 607 "parser.ml"
               : 'rel_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'rel_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cons_expr) in
    Obj.repr(
# 152 "parser.mly"
                            ( RelOp(_1, LtEq, _3) )
# 615 "parser.ml"
               : 'rel_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'rel_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cons_expr) in
    Obj.repr(
# 153 "parser.mly"
                            ( RelOp(_1, GtEq, _3) )
# 623 "parser.ml"
               : 'rel_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'rel_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cons_expr) in
    Obj.repr(
# 154 "parser.mly"
                          ( RelOp(_1, Eq, _3) )
# 631 "parser.ml"
               : 'rel_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'rel_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cons_expr) in
    Obj.repr(
# 155 "parser.mly"
                          ( RelOp(_1, Ne, _3) )
# 639 "parser.ml"
               : 'rel_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'rel_expr) in
    Obj.repr(
# 158 "parser.mly"
             ( _1 )
# 646 "parser.ml"
               : 'assignment))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'rel_expr) in
    Obj.repr(
# 159 "parser.mly"
                             ( Asn(_1, _3) )
# 654 "parser.ml"
               : 'assignment))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'assignment) in
    Obj.repr(
# 162 "parser.mly"
               ( _1 )
# 661 "parser.ml"
               : 'and_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'and_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'assignment) in
    Obj.repr(
# 163 "parser.mly"
                            ( LogOp(_1, And, _3) )
# 669 "parser.ml"
               : 'and_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'and_expr) in
    Obj.repr(
# 166 "parser.mly"
             ( _1 )
# 676 "parser.ml"
               : 'or_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'or_expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'and_expr) in
    Obj.repr(
# 167 "parser.mly"
                        ( LogOp(_1, Or, _3) )
# 684 "parser.ml"
               : 'or_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'or_expr) in
    Obj.repr(
# 170 "parser.mly"
            ( _1 )
# 691 "parser.ml"
               : 'expr))
; (fun __caml_parser_env ->
    Obj.repr(
# 173 "parser.mly"
 ( [] )
# 697 "parser.ml"
               : 'query_args_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'query_args) in
    Obj.repr(
# 174 "parser.mly"
               ( _1 )
# 704 "parser.ml"
               : 'query_args_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'query_arg) in
    Obj.repr(
# 177 "parser.mly"
              ( [_1] )
# 711 "parser.ml"
               : 'query_args))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'query_args) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'query_arg) in
    Obj.repr(
# 178 "parser.mly"
                               ( _3::_1 )
# 719 "parser.ml"
               : 'query_args))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 181 "parser.mly"
         ( _1 )
# 726 "parser.ml"
               : 'query_arg))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 182 "parser.mly"
                   ( RetVar _2 )
# 733 "parser.ml"
               : 'query_arg))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 185 "parser.mly"
      ( [_1] )
# 740 "parser.ml"
               : 'expr_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr_list) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 186 "parser.mly"
                         ( _3::_1 )
# 748 "parser.ml"
               : 'expr_list))
; (fun __caml_parser_env ->
    Obj.repr(
# 189 "parser.mly"
    ( EmptyList )
# 754 "parser.ml"
               : 'expr_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 190 "parser.mly"
         ( _1 )
# 761 "parser.ml"
               : 'expr_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 193 "parser.mly"
    ( List.fold_left add_decl StringMap.empty predefines )
# 767 "parser.ml"
               : 'decls_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'decls) in
    Obj.repr(
# 194 "parser.mly"
          ( _1 )
# 774 "parser.ml"
               : 'decls_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'decl) in
    Obj.repr(
# 197 "parser.mly"
         ( add_decl (List.fold_left add_decl StringMap.empty predefines) _1 )
# 781 "parser.ml"
               : 'decls))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'decls) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'decl) in
    Obj.repr(
# 198 "parser.mly"
               ( add_decl _1 _2 )
# 789 "parser.ml"
               : 'decls))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'rule_decl) in
    Obj.repr(
# 201 "parser.mly"
                        ( _1 )
# 796 "parser.ml"
               : 'decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'truth_decl) in
    Obj.repr(
# 202 "parser.mly"
                         ( _1 )
# 803 "parser.ml"
               : 'decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'literal_list) in
    Obj.repr(
# 206 "parser.mly"
      ( (_1, List.length _3, Pred(List.rev (List.map ptrn_of_lit _3), None)) )
# 811 "parser.ml"
               : 'truth_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'literal) in
    Obj.repr(
# 209 "parser.mly"
            ( [_1] )
# 818 "parser.ml"
               : 'literal_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'literal_list) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'literal) in
    Obj.repr(
# 210 "parser.mly"
                               ( _3::_1 )
# 826 "parser.ml"
               : 'literal_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'literal_list) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 214 "parser.mly"
     ( (_1, List.length _3, Pred(List.rev (List.map ptrn_of_lit _3), Some _6)) )
# 835 "parser.ml"
               : 'rule_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'pattern_list_opt) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 216 "parser.mly"
     ( (_1, List.length _3, Pred(List.rev _3, Some _6)) )
# 844 "parser.ml"
               : 'rule_decl))
; (fun __caml_parser_env ->
    Obj.repr(
# 219 "parser.mly"
    ( [] )
# 850 "parser.ml"
               : 'pattern_list_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'pattern_list) in
    Obj.repr(
# 220 "parser.mly"
                 ( _1 )
# 857 "parser.ml"
               : 'pattern_list_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'pattern) in
    Obj.repr(
# 223 "parser.mly"
            ( [_1] )
# 864 "parser.ml"
               : 'pattern_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'literal_list) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pattern) in
    Obj.repr(
# 224 "parser.mly"
                               ( _3::(List.map ptrn_of_lit _1) )
# 872 "parser.ml"
               : 'pattern_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pattern_list) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pattern) in
    Obj.repr(
# 225 "parser.mly"
                               ( _3::_1 )
# 880 "parser.ml"
               : 'pattern_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pattern_list) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'literal) in
    Obj.repr(
# 226 "parser.mly"
                               ( (ptrn_of_lit _3)::_1 )
# 888 "parser.ml"
               : 'pattern_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 229 "parser.mly"
             ( VarPtrn(_1) )
# 895 "parser.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pattern) in
    Obj.repr(
# 230 "parser.mly"
                          ( ConsPtrn(VarPtrn(_1), _3) )
# 903 "parser.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : string) in
    Obj.repr(
# 231 "parser.mly"
                            ( ConsPtrn(VarPtrn(_1), EmptyPtrn) )
# 910 "parser.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'literal) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pattern) in
    Obj.repr(
# 232 "parser.mly"
                         ( ConsPtrn(ptrn_of_lit _1,_3) )
# 918 "parser.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'literal) in
    Obj.repr(
# 233 "parser.mly"
                           ( ConsPtrn(ptrn_of_lit _1, EmptyPtrn) )
# 925 "parser.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'statements) in
    Obj.repr(
# 236 "parser.mly"
                             ( Block(List.rev _2) )
# 932 "parser.ml"
               : 'block))
; (fun __caml_parser_env ->
    Obj.repr(
# 237 "parser.mly"
                  ( Block([]) )
# 938 "parser.ml"
               : 'block))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'statement) in
    Obj.repr(
# 240 "parser.mly"
              ( [_1] )
# 945 "parser.ml"
               : 'statements))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'statements) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'statement) in
    Obj.repr(
# 241 "parser.mly"
                         ( _2::_1 )
# 953 "parser.ml"
               : 'statements))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 244 "parser.mly"
                   ( ExprStmt _1 )
# 960 "parser.ml"
               : 'statement))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'statement) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'statement) in
    Obj.repr(
# 245 "parser.mly"
                                       ( IfStmt(_1, _3, _5) )
# 969 "parser.ml"
               : 'statement))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'block) in
    Obj.repr(
# 246 "parser.mly"
          ( _1 )
# 976 "parser.ml"
               : 'statement))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'statement) in
    Obj.repr(
# 247 "parser.mly"
                                   ( While(_2,_4) )
# 984 "parser.ml"
               : 'statement))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'statement) in
    Obj.repr(
# 248 "parser.mly"
                                   ( Until(_2,_4) )
# 992 "parser.ml"
               : 'statement))
(* Entry program *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let program (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : Ast.prog)
