/** parser.mly - grammar rules.
    Tabari Alexander tha2105
    COMS W4115 */

%{
  open Ast

  module StringMap = Map.Make(String)

  type decl = {nargs:int; truths:pred list; preds:pred list}

  let rec ptrn_of_lit = function
      IntLit(x) -> IntPtrn(x)
    | AtomLit(x) -> AtomPtrn(x)
    | Var(x) -> VarPtrn(x)
    | EmptyList -> EmptyPtrn
    | _ -> raise(Failure "Unexpected literal in pattern")

  let predefines = 
    [("true", 0, Pred([],
        Some(Call("__true", []))));
     ("false", 0, Pred([],
        Some(Call("__false", []))));
     ("neg", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(Call("__integer", [Var "X"]), And, Call("__neg",[Var "X"; Var "Y"])))));
     ("tointeger", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(Call("__string", [Var "X"]), And, Call("__tointeger",[Var "X"; Var "Y"])))));
     ("tostring", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(Call("__neg",[Var "X"; Var "Y"]))));
     ("exit", 1, Pred([VarPtrn "X"], 
        Some(Call("__exit",[Var "X"]))));
     ("exit", 0, Pred([], 
        Some(Call("__exit",[IntLit 0]))));
     ("true", 1, Pred([VarPtrn "X"], 
        Some(Call("__true",[Var "X"]))));
     ("integer", 1, Pred([VarPtrn "X"], 
        Some(Call("__integer",[Var "X"]))));
     ("string", 1, Pred([VarPtrn "X"], 
        Some(Call("__string",[Var "X"]))));
     ("boolean", 1, Pred([VarPtrn "X"], 
        Some(Call("__boolean",[Var "X"]))));
     ("atom", 1, Pred([VarPtrn "X"], 
        Some(Call("__atom",[Var "X"]))));     
     ("list", 1, Pred([VarPtrn "X"], 
        Some(Call("__list",[Var "X"]))));
     ("list", 1, Pred([EmptyPtrn], None));
     ("not", 1, Pred([VarPtrn "X"], 
        Some(Call("__not",[Call("__true",[Var "X"])]))));
     ("gt", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__gt",[Var "X"; Var "Y"])))));
     ("lt", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__lt",[Var "X"; Var "Y"])))));
     ("ge", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__ge",[Var "X"; Var "Y"])))));
     ("le", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__le",[Var "X"; Var "Y"])))));
     ("eq", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(Call("__eq",[Var "X"; Var "Y"]))));
     ("ne", 2, Pred([VarPtrn "X"; VarPtrn "Y"], 
        Some(Call("__ne",[Var "X"; Var "Y"]))));
     ("cons", 3, Pred([VarPtrn "X"; VarPtrn "Y"; VarPtrn "Z"], 
        Some(LogOp(LogOp(RelOp(Var "Y",Eq, EmptyList),Or,Call("__list", [Var "Y"])), And, Call("__cons",[Var "X"; Var "Y"; Var "Z"])))));
     ("mult", 3, Pred([VarPtrn "X"; VarPtrn "Y"; VarPtrn "Z"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__mult",[Var "X"; Var "Y"; Var "Z"])))));
     ("div", 3, Pred([VarPtrn "X"; VarPtrn "Y"; VarPtrn "Z"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__div",[Var "X"; Var "Y"; Var "Z"])))));
     ("div", 3, Pred([VarPtrn "X"; IntPtrn 0; VarPtrn "Z"],
        Some(LogOp(Call("__print", [StrLit "Divide by zero error"]), And, Call("__exit", [IntLit 1])))));
     ("add", 3, Pred([VarPtrn "X"; VarPtrn "Y"; VarPtrn "Z"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__add",[Var "X"; Var "Y"; Var "Z"])))));
     ("sub", 3, Pred([VarPtrn "X"; VarPtrn "Y"; VarPtrn "Z"], 
        Some(LogOp(LogOp(Call("__integer", [Var "X"]), And, Call("__integer", [Var "Y"])), And, Call("__sub",[Var "X"; Var "Y"; Var "Z"])))));
     ("print", 1, Pred([VarPtrn "X"], 
        Some(Call("__print",[Var "X"]))));
     ("write", 1, Pred([VarPtrn "X"], 
        Some(Call("__write", [Var "X"]))));
     ("write", 1, Pred([ConsPtrn(VarPtrn "H", VarPtrn "T")],
        Some(LogOp(Call("write", [Var "H"]), And, Call("write", [Var "T"])))));
     ("write", 1, Pred([VarPtrn "X"],
        Some(LogOp(Call("__string", [Var "X"]), And, Call("__write", [Var "X"])))))] 

  let add_decl sm = function
    (n, a, p) -> 
      let name = n ^ "/" ^ (string_of_int a) in
      let r = 
        try
          StringMap.find name sm
        with
          Not_found -> {nargs = a; truths=[]; preds=[]} in
      let r = 
        (match p with Pred(_, Some e) -> { r with preds = p::r.preds }
          | Pred(_, None) -> { r with truths = p::r.truths })
      in
      StringMap.add name r sm
       
%}
%token EOF LPAREN RPAREN LBRACE RBRACE SEMICOLON COMMA IMPLY IN LBRACK RBRACK
%token CONS ASSIGN PLUS MINUS TIMES DIVIDE CONS TYPE THEN ELSE NOT AND OR
%token LT GT LTEQ GTEQ EQ NE EMPTYLIST UNTIL WHILE
%token <string> VARIABLE ID STRINGLIT
%token <int> INTLIT

%start program
%type <Ast.prog> program

%%

program:
    decls_opt block 
      { Prog(StringMap.fold (fun n r l -> {name=n; decls = List.rev_append r.truths r.preds; arity = r.nargs}::l) $1 [],$2) }

literal:
    INTLIT { IntLit $1 }
  | ID { AtomLit $1 }
  | STRINGLIT { StrLit $1 }
  | EMPTYLIST { EmptyList }

primary_expr:
    literal { $1 }
  | VARIABLE { Var $1  } 
  | LPAREN expr RPAREN { $2 }
  | LBRACK expr_list RBRACK { List.fold_left (fun e li -> BinOp(li, Cons, e)) EmptyList $2 }
  | query { $1 }
  | LPAREN expr THEN expr ELSE expr_opt RPAREN { IfExpr($2, $4, $6) }

query:
    ID LPAREN query_args_opt RPAREN { Call($1, List.rev $3) }

unary_expr:
    primary_expr { $1 }
  | MINUS unary_expr { Neg $2 }
  | NOT unary_expr { Not $2 }

mult_expr:
    unary_expr { $1 }
  | mult_expr TIMES unary_expr { BinOp($1, Mul, $3) }
  | mult_expr DIVIDE unary_expr { BinOp($1, Div, $3) }

add_expr:
    mult_expr { $1 }
  | add_expr PLUS mult_expr { BinOp($1, Add, $3) }
  | add_expr MINUS mult_expr { BinOp($1, Sub, $3) }

cons_expr:
    add_expr { $1 }
  | add_expr CONS cons_expr { BinOp($1, Cons, $3) }

rel_expr:
    cons_expr { $1 }
  | rel_expr LT cons_expr { RelOp($1, Lt, $3) }
  | rel_expr GT cons_expr { RelOp($1, Gt, $3) }
  | rel_expr LTEQ cons_expr { RelOp($1, LtEq, $3) }
  | rel_expr GTEQ cons_expr { RelOp($1, GtEq, $3) }
  | rel_expr EQ cons_expr { RelOp($1, Eq, $3) }
  | rel_expr NE cons_expr { RelOp($1, Ne, $3) }

assignment:
    rel_expr { $1 }
  | VARIABLE ASSIGN rel_expr { Asn($1, $3) }

and_expr:
    assignment { $1 }
  | and_expr AND assignment { LogOp($1, And, $3) }

or_expr:
    and_expr { $1 }
  | or_expr OR and_expr { LogOp($1, Or, $3) }

expr:
    or_expr { $1 }

query_args_opt:
	{ [] }
  | query_args { $1 }

query_args:
    query_arg { [$1] }
  | query_args COMMA query_arg { $3::$1 }

query_arg:
    expr { $1 }
  | TIMES VARIABLE { RetVar $2 }

expr_list:
	expr { [$1] }
  | expr_list COMMA expr { $3::$1 }

expr_opt:
    { EmptyList }
  | expr { $1 }

decls_opt:
    { List.fold_left add_decl StringMap.empty predefines }
  | decls { $1 }

decls:
    decl { add_decl (List.fold_left add_decl StringMap.empty predefines) $1 }
  | decls decl { add_decl $1 $2 }

decl:
    rule_decl SEMICOLON { $1 }
  | truth_decl SEMICOLON { $1 }

truth_decl:
    ID LPAREN literal_list RPAREN 
      { ($1, List.length $3, Pred(List.rev (List.map ptrn_of_lit $3), None)) }

literal_list:
    literal { [$1] }
  | literal_list COMMA literal { $3::$1 }

rule_decl:
    ID LPAREN literal_list RPAREN IMPLY expr 
     { ($1, List.length $3, Pred(List.rev (List.map ptrn_of_lit $3), Some $6)) }
  | ID LPAREN pattern_list_opt RPAREN IMPLY expr
     { ($1, List.length $3, Pred(List.rev $3, Some $6)) }

pattern_list_opt:
    { [] }
  | pattern_list { $1 }

pattern_list:
    pattern { [$1] }
  | literal_list COMMA pattern { $3::(List.map ptrn_of_lit $1) }
  | pattern_list COMMA pattern { $3::$1 }
  | pattern_list COMMA literal { (ptrn_of_lit $3)::$1 }

pattern:
    VARIABLE { VarPtrn($1) }
  | VARIABLE CONS pattern { ConsPtrn(VarPtrn($1), $3) }
  | VARIABLE CONS EMPTYLIST { ConsPtrn(VarPtrn($1), EmptyPtrn) }
  | literal CONS pattern { ConsPtrn(ptrn_of_lit $1,$3) }
  | literal CONS EMPTYLIST { ConsPtrn(ptrn_of_lit $1, EmptyPtrn) }

block:
    LBRACE statements RBRACE { Block(List.rev $2) }
  | LBRACE RBRACE { Block([]) }

statements:
    statement { [$1] }
  | statements statement { $2::$1 }

statement:
    expr SEMICOLON { ExprStmt $1 }
  | expr THEN statement ELSE statement { IfStmt($1, $3, $5) }
  | block { $1 }
  | WHILE expr SEMICOLON statement { While($2,$4) }
  | UNTIL expr SEMICOLON statement { Until($2,$4) }

