(** sast.mli - This file defines the types for a Syntax-checked Abstract Syntax 
   Tree that is produced by the parser.
   Tabari Alexander tha2105
   COMS W4115
*)

(** The set of operators for an arithmetic op *)
type operator = Add | Sub | Mul | Div | Cons

(** The set of operators for a relation op *) 
type relop = Eq | Ne | Lt | Gt | LtEq | GtEq

(** The set of operators for a logic op *)
type logicop = And | Or

(** Expressions that mostly map to the expressions in the language's grammar. *)
type expr = 
    IntLit of int
  | AtomLit of string
  | BoolLit of int
  | EmptyList
  | StrLit of string
  | Call of string * expr list
  | BinOp of expr * Ast.operator * expr
  | RelOp of expr * Ast.relop * expr
  | LogOp of expr * Ast.logicop * expr
  | AsnEq of expr * expr
  | Var of string
  | RetVar of string
  | Asn of string * expr
  | SetAsn of string * string
  | IfExpr of expr * expr * expr
  | Not of expr
  | Seq of expr list
  | Clear of string
  | TestListExpr of expr
  | HeadExpr of expr
  | TailExpr of expr
  | Ignore of expr

type decl = Decl of string * int * expr

type stmt = 
    ExprStmt of expr
  | IfStmt of expr * stmt * stmt
  | Block of stmt list
  | While of expr * stmt
  | Until of expr * stmt

type prog =
    Prog of decl list * stmt

(** An exception which signifies that some sort of syntax error has been
    detected. *)
exception SyntaxError of string

(** var translate : Ast.prog -> prog *)
val translate : Ast.prog -> prog

