(** scanner.mll - the lexical analyzer. 
   Tabari Alexander tha2105
   COMS W4115
*)
{ open Parser 
  exception ScannerError of string
}
let ucletter = ['A'-'Z']
let lcletter = ['a'-'z' '_']
let letter = ucletter|lcletter
let digit = ['0'-'9']
rule token = parse 
	[' ' '\t' '\r' '\n']+ { token lexbuf }
  | '+'                 { PLUS }
  | '-'                 { MINUS }
  | '*'                 { TIMES }
  | '/'                 { DIVIDE }
  | '='                 { ASSIGN }
  | '('                 { LPAREN }
  | ')'                 { RPAREN }
  | '{'                 { LBRACE }
  | '}'                 { RBRACE }
  | '['                 { LBRACK }
  | ']'                 { RBRACK }
  | '|'                 { CONS }
  | '!'                 { NOT }
  | '?'                 { THEN }
  | ':'                 { ELSE }
  | '>'                 { GT }
  | '<'                 { LT }
  | '&'                 { AND }
  | ';'                 { SEMICOLON }
  | ','                 { COMMA }
  | ">="                { GTEQ }
  | "<="                { LTEQ }
  | "||"                { OR }
  | "=="                { EQ }
  | "!="                { NE }
  | "[]"                { EMPTYLIST }
  | "[["                { comment lexbuf } (* Comment *)
  | "->"                { IMPLY }
  | "until"             { UNTIL }
  | "while"             { WHILE }
  | ucletter(letter|digit)* as lit { VARIABLE lit }
  | lcletter(letter|digit)* as lit { ID lit }
  | ['1'-'9']digit*|'0' as lit { INTLIT(int_of_string lit) }
  | '"' (([^ '"' '\\'] | '\\'['\\' '"' 't' 'n'])* as s) '"' { STRINGLIT s }
  | eof                 { EOF }
  | _ as ch             { raise(ScannerError("Unexpected character: " ^ (Char.escaped ch))) }
(* Comment - it just gobbles everything up and tosses it. *)
and comment = parse 
	"]]"                    { token lexbuf }
  | eof                 { raise(ScannerError "Unexpected end of file while handling a comment!") }
  | _                       { comment lexbuf }
